//HDL Example 8-2
//------------------------------------------   
//RTL description of design example (Fig.8-9)
module Example_RTL (S,CLK,Clr,E,F,A);
//Specify inputs and outputs 
//See block diagram Fig. 8-10
   input S,CLK,Clr;
   output E,F;
   output [4:1] A;
//Specify system registers
   reg [4:1] A;                //A register
   reg E, F;                   //E and F flip-flops
   reg [1:0] pstate, nstate;   //control register
//Encode the states
   parameter T0 = 2'b00, T1 = 2'b01, T2 = 2'b11;
//State transition for control logic
//See state diagram Fig. 8-11(a) 
   always @(posedge CLK or negedge Clr)
      if (~Clr) pstate = T0;    //Initial state
      else pstate <= nstate;    //Clocked operations
   always @ (S or A or pstate)
       case (pstate)
         T0: if(S) nstate = T1;
         T1: if(A[3] & A[4]) nstate = T2;
         T2: nstate = T0;
         default: nstate = T0;
       endcase 
//Register transfer operatons 
//See list of operations Fig.8-11(b)
   always @(posedge CLK)
       case (pstate)
         T0: if(S)
              begin
                 A <= 4'b0000;
                 F <= 1'b0;
              end
         T1: 
              begin
                 A <= A + 1'b1;
                 if (A[3]) E <= 1'b1;
                 else  E <= 1'b0;
              end
         T2:  F <= 1'b1;
      endcase
endmodule

