//HDL Example 8-5
//--------------------------------------      
//RTL description of binary multiplier
//Block diagram Fig.8-13 and ASM chart Fig.8-14 
//n = 5 to compare with Table 8-4
module mltp(S,CLK,Clr,Binput,Qinput,C,A,Q,P,pstate);
   input S,CLK,Clr;
   input [4:0] Binput,Qinput;          //Data inputs
   output C;
   output [4:0] A,Q;
   output [2:0] P;
   output [1:0] pstate;
//System registers
   reg C;
   reg [4:0] A,Q,B;
   reg [2:0] P;
   reg [1:0] pstate, nstate;           //control register
   parameter T0=2'b00, T1=2'b01, T2=2'b10, T3=2'b11;
//Combinational circuit
   wire Z;
   assign Z = ~|P;                     //Check for zero
//State transition for control
//See state diagram Fig. 8-15(a)
   always @(negedge CLK or negedge Clr)
     if (~Clr) pstate = T0;
     else pstate <= nstate;
   always @(S or Z or pstate)
     case (pstate)
       T0: if (S) nstate = T1;
       T1: nstate = T2;
       T2: nstate = T3;
       T3: if (Z) nstate = T0;
           else   nstate = T2;
     endcase
//Register transfer operations 
//See register operation Fig.8-15(b)
   always @(negedge CLK)
     case (pstate)
       T0: B <= Binput;                //Input multiplicand
       T1: begin
             A <= 5'b00000;
             C <= 1'b0;
             P <= 3'b101;              //Initialize counter to n=5
             Q <= Qinput;              //Input multiplier 
           end 
       T2: begin
             P <= P - 3'b001;          //Decrement counter
             if (Q[0]) 
             {C,A} <= A + B;           //Add multiplicand
           end
       T3: begin
             C <= 1'b0;                //Clear C
             A <= {C,A[4:1]};          //Shift right A 
             Q <= {A[0],Q[4:1]};       //Shift right Q
           end 
     endcase
endmodule
